@@ -23,33 +23,38 @@ pub enum Behavior<T> {
         Box<Node<T>>,
     ),
 
-    // TODO: store cursor here + continue from where left off
     Sequence(usize, Vec<Node<T>>),
+    Select(usize, Vec<Node<T>>),
+
     Action(String, fn(&mut T) -> Status),
     ActionSuccess(String, fn(&mut T) -> ()),
 
     StatefulAction(String, Box<dyn StatefulAction<T>>),
     // StatefulAction(String, fn(&mut T, &P) -> Status),
 
-    // Select(Vec<Behavior<T>>),
-
     // Invert(Box<Behavior<T>>),
     // AlwaysSucceed(Box<Behavior<T>>),
     // Condition(Box<dyn Fn(f64, &mut T, &P) -> bool>, Box<Behavior<T>>),
     // WaitForever,
     // Action(T),
-    // While(Box<Behavior<T>>, Box<Behavior<T>>),
+    While(fn(&T) -> bool, Box<Node<T>>),
 }
 
 fn sequence<T>(
     delta: f64,
     context: &mut T,
-    _is_sequence: bool,
+    is_sequence: bool,
     current: &mut usize,
     xs: &mut Vec<Node<T>>,
 ) -> (Status, DebugRepr) {
+    let (status_positive, status_negative) = if is_sequence {
+        (Status::Success, Status::Failure)
+    } else {
+        (Status::Failure, Status::Success)
+    };
+
     let mut repr_string = String::new();
-    let mut status = Status::Success;
+    let mut status = status_positive;
     let mut child_repr = None;
 
     let len = xs.len();
@@ -65,28 +70,34 @@ fn sequence<T>(
         if x.status == Status::Success || x.status == Status::Failure {
             x.behavior.reset();
         }
+        let (res, repr) = x.tick(delta, context);
 
-        match x.tick(delta, context) {
-            (Status::Success, repr) => {
-                *current += 1;
-                repr_string += "+";
-                child_repr = Some(repr);
-            }
-            (Status::Failure, repr) => {
-                status = Status::Failure;
-                repr_string += "-";
-                child_repr = Some(repr);
-                break;
-                // return (Status::Failure, DebugRepr::new("Sequence", Status::Failure))
-            }
-            (Status::Running, repr) => {
-                status = Status::Running;
-                repr_string += ".";
-                child_repr = Some(repr);
-                break;
-                // return (Status::Running, DebugRepr::new("Sequence", Status::Running))
-            }
+        if res == status_positive {
+            *current += 1;
+            repr_string += "+";
+            child_repr = Some(repr);
+        } else if res == status_negative {
+            status = status_negative;
+            repr_string += "-";
+            child_repr = Some(repr);
+            break;
+        } else {
+            status = Status::Running;
+            repr_string += ".";
+            child_repr = Some(repr);
+            break;
         }
+
+        // match x.tick(delta, context) {
+        //     (Status::Success, repr) => {
+        //     }
+        //     (Status::Failure, repr) => {
+        //         // return (Status::Failure, DebugRepr::new("Sequence", Status::Failure))
+        //     }
+        //     (Status::Running, repr) => {
+        //         // return (Status::Running, DebugRepr::new("Sequence", Status::Running))
+        //     }
+        // }
     }
 
     let mut repr = DebugRepr::new(
@@ -172,6 +183,10 @@ impl<T> Behavior<T> {
                 return sequence(delta, context, true, current, xs)
             }
 
+            Behavior::Select(ref mut current, xs) => {
+                return sequence(delta, context, false, current, xs)
+            }
+
             Behavior::Action(name, action) => {
                 let status = action(context);
                 return (status, DebugRepr::new(name, Cursor::Leaf, status));
@@ -189,8 +204,16 @@ impl<T> Behavior<T> {
             Behavior::StatefulAction(name, action) => {
                 let status = action.tick(context);
                 return (status, DebugRepr::new(name, Cursor::Leaf, status));
-            } // Behavior::Invert(b) => match b.tick(delta, context).0 {
-              //     Status::Success => Status::Failure,
+            }
+
+            Behavior::While(cond, behavior) => {
+                if cond(context) {
+                    return behavior.tick(delta, context);
+                } else {
+                    let status = Status::Failure;
+                    return (status, DebugRepr::new("while", Cursor::Leaf, status));
+                }
+            } //     Status::Success => Status::Failure,
               //     Status::Failure => Status::Success,
               //     Status::Running => Status::Running,
               // },
@@ -243,29 +266,6 @@ impl<T> Behavior<T> {
             _ => {}
         }
     }
-
-    pub fn to_debug(&self) -> TreeRepr {
-        match self {
-            Behavior::Wait { curr, max } => {
-                TreeRepr::new("Wait", vec![]).with_detail(format!("curr={}, max={}", curr, max))
-            }
-            Behavior::Cond(name, _cond, a, b) => {
-                TreeRepr::new("Cond", vec![a.behavior.to_debug(), b.behavior.to_debug()])
-                    .with_detail(name.clone())
-            }
-            Behavior::Sequence(_, seq) => TreeRepr::new(
-                "Sequence",
-                seq.iter().map(|x| x.behavior.to_debug()).collect(),
-            ),
-            Behavior::Action(name, _) => TreeRepr::new("Action", vec![]).with_detail(name.clone()),
-            Behavior::ActionSuccess(name, _) => {
-                TreeRepr::new("ActionSuccess", vec![]).with_detail(name.clone())
-            }
-            Behavior::StatefulAction(name, _) => {
-                TreeRepr::new("StatefulAction", vec![]).with_detail(name.clone())
-            }
-        }
-    }
 }
 
 impl<T> core::fmt::Debug for Behavior<T> {
